#include <stdio.h>
#include <stdlib.h>
enum { N = 4 };

void func1(const int mat[][N]);
void func2(const int (*mat)[N]);
void func3(const int *mat);
void func4(int **mat);

int main(void) {
	int mat[N][N];
	for(int i = 0; i < N; i++) {
		for(int j = 0; j < N; j++) {
			mat[i][j] = 10 * i + j;
		}
	}

	func1(mat);
	func2(mat);
	func3((int*)mat);
	func4((int**)mat);

	return EXIT_SUCCESS;
}

void func1(const int mat[][N]) {
	printf("Declare as matrix, explicitly specify 2nd dimension:\n");
	for(int i = 0; i < N; i++) {
		for(int j = 0; j < N; j++) {
			printf("%5.2d", mat[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

//	Method #2
//	Pointer to array, second dimension is explicitly specified.
//	The declaration int (*mat)[N] says that mat is a pointer
//	to an array of 'N' ints.

void func2(const int (*mat)[N]) {
	printf("Declare as pointer to row, explicitly specify 2nd N:\n");
	for(int i = 0; i < N; i++) {
		for(int j = 0; j < N; j++) {
			printf("%5.2d", mat[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

//	Method #3
//	Using a single pointer, the array is "flattened"
//	With this method you can create general-purpose routines.
//	The dimensions don't appear in any declaration, so you
//	can add them to the formal argument list.

//	The manual array indexing will probably slow down execution.

void func3(const int *mat)	{
	printf("Declare as single-pointer, manual offset computation:\n");
	for(int i = 0; i < N; i++) {
		for(int j = 0; j < N; j++) {
			printf("%5.2d", *(mat + N * i + j));
		}
		printf("\n");
	}
	printf("\n");
}

//	Method #4 (double pointer, using an auxiliary array of pointers).
//	With this method you can create general-purpose routines,
//	if you allocate "index" at run-time.

//	Add the dimensions to the formal argument list.

void func4(int **mat) {
	int *index[N];	//	index[i] contains address of the ith row
	index[0] = (int*) mat;
	for (int i = 1; i < N; i++)
		index[i] = index[i-1] + N;
	printf("Declare as double-pointer, use auxiliary pointer array:\n");
	//	from now on use index instead of mat
	for(int i = 0; i < N; i++) {
		for(int j = 0; j < N; j++) {
			printf("%5.2d", index[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

